pub mod db;
use actix_web::{get,post, web, HttpResponse, Responder, Result};
use serde::{Deserialize, Serialize};

pub struct LoyaltyService {
    pub db: db::DbService,
}

#[derive(Serialize)]
struct ArrayResponse<T> {
    result: Vec<T>,
}


#[derive(Debug, Serialize, Deserialize)]
pub struct AddColorRequest {
    pub name: String,
}

#[get("/cats")]
pub async fn get_cats(ctx: web::Data<LoyaltyService>) -> Result<impl Responder> {
    let cats = ctx.db.get_cats();
    let res = match cats {
        Ok(v) => ArrayResponse{ result: v },
        Err(_err) => ArrayResponse { result: vec![] },
    };
    return Ok(HttpResponse::Ok().json(res));
}

#[get("/colors")]
pub async fn get_colors(ctx: web::Data<LoyaltyService>) -> Result<impl Responder> {
    let colors = ctx.db.get_colors();
    let res = match colors {
        Ok(v) => ArrayResponse { result: v },
        Err(_err) => ArrayResponse { result: vec![] },
    };
    return Ok(HttpResponse::Ok().json(res));
}

#[derive(Debug, Serialize, Deserialize)]
pub struct AddCatRequest {
    pub name: String,
    pub color: String,
}


#[post("/add/cat")]
pub async fn add_cat(
    ctx: web::Data<LoyaltyService>,
    cat: web::Json<AddCatRequest>,
) -> Result<impl Responder> {
    let _cat = cat.into_inner();
    let cat = ctx.db.add_cat(_cat.name, _cat.color);
    let res = match cat {
        Ok(v) => ArrayResponse { result: vec![v] },
        Err(_err) => panic!("{:?}", _err),
    };
    return Ok(HttpResponse::Ok().json(res));
}

#[post("/add/color")]
pub async fn add_color(
    ctx: web::Data<LoyaltyService>,
    cat: web::Json<AddColorRequest>,
) -> Result<impl Responder> {
    let _color = cat.into_inner();
    let cat = ctx.db.add_color(_color.name);
    let res = match cat {
        Ok(v) => ArrayResponse { result: vec![v] },
        Err(_err) => panic!("{:?}", _err),
    };
    return Ok(HttpResponse::Ok().json(res));
}

